@@ -1,3 +1,25 @@
+// -*- C++ -*-
+/* Copyright (C) 2000, 2003, 2009
+   Free Software Foundation, Inc.
+     Written by Francisco Andrs Verd <pandres@dragonet.es> with many ideas
+     taken from the other groff drivers.
+
+
+This file is part of groff.
+
+groff is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free
+Software Foundation, either version 3 of the License, or
+(at your option) any later version.
+
+groff is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program. If not, see <http://www.gnu.org/licenses/>. */
+
 // Definition of the WP54 character set
 
 unsigned char symset[] = {
