@@ -1,22 +1,22 @@
 // -*- C++ -*-
-/* Copyright (C) 1992, 2000, 2001, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1992, 2000, 2001, 2002, 2005, 2009
+     Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2, or (at your option) any later
-version.
+Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 groff is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
-You should have received a copy of the GNU General Public License along
-with groff; see the file COPYING.  If not, write to the Free Software
-Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+You should have received a copy of the GNU General Public License
+along with this program. If not, see <http://www.gnu.org/licenses/>. */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -58,6 +58,10 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #define O_RDONLY 0
 #endif
 
+#ifndef F_OK
+#define F_OK 0
+#endif
+
 #ifndef HAVE_ISATTY
 #define isatty(n) (1)
 #endif
