@@ -2,19 +2,20 @@
 
 // <groff_src_dir>/src/include/printer.h
 
-/* Copyright (C) 1989, 1990, 1991, 1992, 2001, 2002, 2003
+/* Copyright (C) 1989, 1990, 1991, 1992, 2001, 2002, 2003, 2004, 2006,
+                 2009
    Free Software Foundation, Inc.
 
    Written by James Clark (jjc@jclark.com)
 
-   Last update: 12 Apr 2002
+   Last update: 5 Jan 2009
 
    This file is part of groff.
 
    groff is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    groff is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,9 +23,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with groff; see the file COPYING.  If not, write to the Free
-   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.
+   along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* Description
@@ -49,7 +48,7 @@ struct environment {
   color *fill;
 };
 
-struct font;
+class font;
 
 struct font_pointer_list {
   font *p;
@@ -62,27 +61,25 @@ class printer {
 public:
   printer();
   virtual ~printer();
-  void load_font(int i, const char *name);
-  void set_ascii_char(unsigned char c, const environment *env,
-		      int *widthp = 0);
-  void set_special_char(const char *nm, const environment *env,
-			int *widthp = 0);
-  virtual void set_numbered_char(int n, const environment *env,
-				 int *widthp = 0);
-  int set_char_and_width(const char *nm, const environment *env,
-			 int *widthp, font **f);
-  font *get_font_from_index(int fontno);
-  virtual void draw(int code, int *p, int np, const environment *env);
+  void load_font(int, const char *);
+  void set_ascii_char(unsigned char, const environment *, int * = 0);
+  void set_special_char(const char *, const environment *, int * = 0);
+  virtual void set_numbered_char(int, const environment *, int * = 0);
+  glyph *set_char_and_width(const char *, const environment *,
+			    int *, font **);
+  font *get_font_from_index(int);
+  virtual void draw(int, int *, int, const environment *);
   // perform change of line color (text, outline) in the print-out 
-  virtual void change_color(const environment * const env);
+  virtual void change_color(const environment * const);
   // perform change of fill color in the print-out 
-  virtual void change_fill_color(const environment * const env);
+  virtual void change_fill_color(const environment * const);
   virtual void begin_page(int) = 0;
-  virtual void end_page(int page_length) = 0;
-  virtual font *make_font(const char *nm);
+  virtual void end_page(int) = 0;
+  virtual font *make_font(const char *);
   virtual void end_of_line();
-  virtual void special(char *arg, const environment *env,
-		       char type = 'p');
+  virtual void special(char *, const environment *, char = 'p');
+  virtual void devtag(char *, const environment *, char = 'p');
+
 protected:
   font_pointer_list *font_list;
   font **font_table;
@@ -97,8 +94,8 @@ protected:
 
 private:
   font *find_font(const char *);
-  virtual void set_char(int index, font *f, const environment *env,
-			int w, const char *name) = 0;
+  virtual void set_char(glyph *, font *, const environment *, int,
+			const char *) = 0;
 };
 
 printer *make_printer();
