@@ -1,5 +1,6 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001, 2002, 2003, 2004
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001, 2002, 2003, 2004, 2005,
+                 2006, 2008, 2009, 2010
    Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
@@ -7,17 +8,16 @@ This file is part of groff.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2, or (at your option) any later
-version.
+Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 groff is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
-You should have received a copy of the GNU General Public License along
-with groff; see the file COPYING.  If not, write to the Free Software
-Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+You should have received a copy of the GNU General Public License
+along with this program. If not, see <http://www.gnu.org/licenses/>. */
 
 #include "lib.h"
 
@@ -25,10 +25,12 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include <assert.h>
 #include <math.h>
 #include <stdlib.h>
+#include <wchar.h>
 #include "errarg.h"
 #include "error.h"
 #include "cset.h"
 #include "font.h"
+#include "unicode.h"
 #include "paper.h"
 
 const char *const WS = " \t\n\r";
@@ -46,12 +48,12 @@ struct font_char_metric {
 };
 
 struct font_kern_list {
-  int i1;
-  int i2;
+  glyph *glyph1;
+  glyph *glyph2;
   int amount;
   font_kern_list *next;
 
-  font_kern_list(int, int, int, font_kern_list * = 0);
+  font_kern_list(glyph *, glyph *, int, font_kern_list * = 0);
 };
 
 struct font_widths_cache {
@@ -71,6 +73,7 @@ struct text_file {
   int lineno;
   int size;
   int skip_comments;
+  int silent;
   char *buf;
   text_file(FILE *fp, char *p);
   ~text_file();
@@ -82,7 +85,7 @@ struct text_file {
 };
 
 text_file::text_file(FILE *p, char *s) 
-: fp(p), path(s), lineno(0), size(0), skip_comments(1), buf(0)
+: fp(p), path(s), lineno(0), size(0), skip_comments(1), silent(0), buf(0)
 {
 }
 
@@ -141,20 +144,63 @@ void text_file::error(const char *format,
 		      const errarg &arg2,
 		      const errarg &arg3)
 {
-  error_with_file_and_line(path, lineno, format, arg1, arg2, arg3);
+  if (!silent)
+    error_with_file_and_line(path, lineno, format, arg1, arg2, arg3);
 }
 
+int glyph_to_unicode(glyph *g)
+{
+  const char *nm = glyph_to_name(g);
+  if (nm != NULL) {
+    // ASCII character?
+    if (nm[0] == 'c' && nm[1] == 'h' && nm[2] == 'a' && nm[3] == 'r'
+	&& (nm[4] >= '0' && nm[4] <= '9')) {
+      int n = (nm[4] - '0');
+      if (nm[5] == '\0')
+	return n;
+      if (n > 0 && (nm[5] >= '0' && nm[5] <= '9')) {
+	n = 10*n + (nm[5] - '0');
+	if (nm[6] == '\0')
+	  return n;
+	if (nm[6] >= '0' && nm[6] <= '9') {
+	  n = 10*n + (nm[6] - '0');
+	  if (nm[7] == '\0' && n < 128)
+	    return n;
+	}
+      }
+    }
+    // Unicode character?
+    if (check_unicode_name(nm)) {
+      char *ignore;
+      return (int)strtol(nm + 1, &ignore, 16);
+    }
+    // If `nm' is a single letter `x', the glyph name is `\x'.
+    char buf[] = { '\\', '\0', '\0' };
+    if (nm[1] == '\0') {
+      buf[1] = nm[0];
+      nm = buf;
+    }
+    // groff glyphs that map to Unicode?
+    const char *unicode = glyph_name_to_unicode(nm);
+    if (unicode != NULL && strchr(unicode, '_') == NULL) {
+      char *ignore;
+      return (int)strtol(unicode, &ignore, 16);
+    }
+  }
+  return -1;
+}
 
 /* font functions */
 
 font::font(const char *s)
-: ligatures(0), kern_hash_table(0), space_width(0), ch_index(0), nindices(0),
-  ch(0), ch_used(0), ch_size(0), special(0), widths_cache(0)
+: ligatures(0), kern_hash_table(0), space_width(0), special(0),
+  ch_index(0), nindices(0), ch(0), ch_used(0), ch_size(0), widths_cache(0)
 {
   name = new char[strlen(s) + 1];
   strcpy(name, s);
   internalname = 0;
   slant = 0.0;
+  zoom = 0;
   // load();			// for testing
 }
 
@@ -192,20 +238,34 @@ static int scale_round(int n, int x, int y)
   if (x == 0)
     return 0;
   if (n >= 0) {
-    if (n <= (INT_MAX - y2)/x)
-      return (n*x + y2)/y;
-    return int(n*double(x)/double(y) + .5);
+    if (n <= (INT_MAX - y2) / x)
+      return (n * x + y2) / y;
+    return int(n * double(x) / double(y) + .5);
   }
   else {
-    if (-(unsigned)n <= (-(unsigned)INT_MIN - y2)/x)
-      return (n*x - y2)/y;
-    return int(n*double(x)/double(y) - .5);
+    if (-(unsigned)n <= (-(unsigned)INT_MIN - y2) / x)
+      return (n * x - y2) / y;
+    return int(n * double(x) / double(y) - .5);
   }
 }
 
+static int scale_round(int n, int x, int y, int z)
+{
+  assert(x >= 0 && y > 0 && z > 0);
+  if (x == 0)
+    return 0;
+  if (n >= 0)
+    return int((n * double(x) / double(y)) * (double(z) / 1000.0) + .5);
+  else
+    return int((n * double(x) / double(y)) * (double(z) / 1000.0) - .5);
+}
+
 inline int font::scale(int w, int sz)
 {
-  return sz == unitwidth ? w : scale_round(w, sz, unitwidth);
+  if (zoom)
+    return scale_round(w, sz, unitwidth, zoom);
+  else
+    return sz == unitwidth ? w : scale_round(w, sz, unitwidth);
 }
 
 int font::unit_scale(double *value, char unit)
@@ -236,15 +296,29 @@ int font::unit_scale(double *value, char unit)
   return 0;
 }
 
-int font::get_skew(int c, int point_size, int sl)
+int font::get_skew(glyph *g, int point_size, int sl)
 {
-  int h = get_height(c, point_size);
-  return int(h*tan((slant+sl)*PI/180.0) + .5);
+  int h = get_height(g, point_size);
+  return int(h * tan((slant + sl) * PI / 180.0) + .5);
 }
 
-int font::contains(int c)
+int font::contains(glyph *g)
 {
-  return c >= 0 && c < nindices && ch_index[c] >= 0;
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  // Explicitly enumerated glyph?
+  if (idx < nindices && ch_index[idx] >= 0)
+    return 1;
+  if (is_unicode) {
+    // Unicode font
+    // ASCII or Unicode character, or groff glyph name that maps to Unicode?
+    if (glyph_to_unicode(g) >= 0)
+      return 1;
+    // Numbered character?
+    if (glyph_to_number(g) >= 0)
+      return 1;
+  }
+  return 0;
 }
 
 int font::is_special()
@@ -266,65 +340,149 @@ font_widths_cache::~font_widths_cache()
   a_delete width;
 }
 
-int font::get_width(int c, int point_size)
+int font::get_width(glyph *g, int point_size)
 {
-  assert(c >= 0 && c < nindices);
-  int i = ch_index[c];
-  assert(i >= 0);
-
-  if (point_size == unitwidth)
-    return ch[i].width;
-
-  if (!widths_cache)
-    widths_cache = new font_widths_cache(point_size, ch_size);
-  else if (widths_cache->point_size != point_size) {
-    font_widths_cache **p;
-    for (p = &widths_cache; *p; p = &(*p)->next)
-      if ((*p)->point_size == point_size)
-	break;
-    if (*p) {
-      font_widths_cache *tem = *p;
-      *p = (*p)->next;
-      tem->next = widths_cache;
-      widths_cache = tem;
-    }
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  int real_size;
+  if (!zoom)
+    real_size = point_size;
+  else
+  {
+    if (point_size <= (INT_MAX - 500) / zoom)
+      real_size = (point_size * zoom + 500) / 1000;
     else
-      widths_cache = new font_widths_cache(point_size, ch_size, widths_cache);
+      real_size = int(point_size * double(zoom) / 1000.0 + .5);
   }
-  int &w = widths_cache->width[i];
-  if (w < 0)
-    w = scale(ch[i].width, point_size);
-  return w;
+  if (idx < nindices && ch_index[idx] >= 0) {
+    // Explicitly enumerated glyph
+    int i = ch_index[idx];
+    if (real_size == unitwidth || font::unscaled_charwidths)
+      return ch[i].width;
+
+    if (!widths_cache)
+      widths_cache = new font_widths_cache(real_size, ch_size);
+    else if (widths_cache->point_size != real_size) {
+      font_widths_cache **p;
+      for (p = &widths_cache; *p; p = &(*p)->next)
+	if ((*p)->point_size == real_size)
+	  break;
+      if (*p) {
+	font_widths_cache *tem = *p;
+	*p = (*p)->next;
+	tem->next = widths_cache;
+	widths_cache = tem;
+      }
+      else
+	widths_cache = new font_widths_cache(real_size, ch_size,
+					     widths_cache);
+    }
+    int &w = widths_cache->width[i];
+    if (w < 0)
+      w = scale(ch[i].width, point_size);
+    return w;
+  }
+  if (is_unicode) {
+    // Unicode font
+    int width = 24; // XXX: Add a request to override this.
+    int w = wcwidth(get_code(g));
+    if (w > 1)
+      width *= w;
+    if (real_size == unitwidth || font::unscaled_charwidths)
+      return width;
+    else
+      return scale(width, point_size);
+  }
+  abort();
 }
 
-int font::get_height(int c, int point_size)
+int font::get_height(glyph *g, int point_size)
 {
-  assert(c >= 0 && c < nindices && ch_index[c] >= 0);
-  return scale(ch[ch_index[c]].height, point_size);
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  if (idx < nindices && ch_index[idx] >= 0) {
+    // Explicitly enumerated glyph
+    return scale(ch[ch_index[idx]].height, point_size);
+  }
+  if (is_unicode) {
+    // Unicode font
+    return 0;
+  }
+  abort();
 }
 
-int font::get_depth(int c, int point_size)
+int font::get_depth(glyph *g, int point_size)
 {
-  assert(c >= 0 && c < nindices && ch_index[c] >= 0);
-  return scale(ch[ch_index[c]].depth, point_size);
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  if (idx < nindices && ch_index[idx] >= 0) {
+    // Explicitly enumerated glyph
+    return scale(ch[ch_index[idx]].depth, point_size);
+  }
+  if (is_unicode) {
+    // Unicode font
+    return 0;
+  }
+  abort();
 }
 
-int font::get_italic_correction(int c, int point_size)
+int font::get_italic_correction(glyph *g, int point_size)
 {
-  assert(c >= 0 && c < nindices && ch_index[c] >= 0);
-  return scale(ch[ch_index[c]].italic_correction, point_size);
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  if (idx < nindices && ch_index[idx] >= 0) {
+    // Explicitly enumerated glyph
+    return scale(ch[ch_index[idx]].italic_correction, point_size);
+  }
+  if (is_unicode) {
+    // Unicode font
+    return 0;
+  }
+  abort();
 }
 
-int font::get_left_italic_correction(int c, int point_size)
+int font::get_left_italic_correction(glyph *g, int point_size)
 {
-  assert(c >= 0 && c < nindices && ch_index[c] >= 0);
-  return scale(ch[ch_index[c]].pre_math_space, point_size);
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  if (idx < nindices && ch_index[idx] >= 0) {
+    // Explicitly enumerated glyph
+    return scale(ch[ch_index[idx]].pre_math_space, point_size);
+  }
+  if (is_unicode) {
+    // Unicode font
+    return 0;
+  }
+  abort();
 }
 
-int font::get_subscript_correction(int c, int point_size)
+int font::get_subscript_correction(glyph *g, int point_size)
 {
-  assert(c >= 0 && c < nindices && ch_index[c] >= 0);
-  return scale(ch[ch_index[c]].subscript_correction, point_size);
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  if (idx < nindices && ch_index[idx] >= 0) {
+    // Explicitly enumerated glyph
+    return scale(ch[ch_index[idx]].subscript_correction, point_size);
+  }
+  if (is_unicode) {
+    // Unicode font
+    return 0;
+  }
+  abort();
+}
+
+void font::set_zoom(int factor)
+{
+  assert(factor >= 0);
+  if (factor == 1000)
+    zoom = 0;
+  else
+    zoom = factor;
+}
+
+int font::get_zoom()
+{
+  return zoom;
 }
 
 int font::get_space_width(int point_size)
@@ -332,33 +490,35 @@ int font::get_space_width(int point_size)
   return scale(space_width, point_size);
 }
 
-font_kern_list::font_kern_list(int c1, int c2, int n, font_kern_list *p)
-: i1(c1), i2(c2), amount(n), next(p)
+font_kern_list::font_kern_list(glyph *g1, glyph *g2, int n, font_kern_list *p)
+: glyph1(g1), glyph2(g2), amount(n), next(p)
 {
 }
 
-inline int font::hash_kern(int i1, int i2)
+inline int font::hash_kern(glyph *g1, glyph *g2)
 {
-  int n = ((i1 << 10) + i2) % KERN_HASH_TABLE_SIZE;
+  int n = ((glyph_to_index(g1) << 10) + glyph_to_index(g2))
+	  % KERN_HASH_TABLE_SIZE;
   return n < 0 ? -n : n;
 }
 
-void font::add_kern(int i1, int i2, int amount)
+void font::add_kern(glyph *g1, glyph *g2, int amount)
 {
   if (!kern_hash_table) {
     kern_hash_table = new font_kern_list *[int(KERN_HASH_TABLE_SIZE)];
     for (int i = 0; i < KERN_HASH_TABLE_SIZE; i++)
       kern_hash_table[i] = 0;
   }
-  font_kern_list **p = kern_hash_table + hash_kern(i1, i2);
-  *p = new font_kern_list(i1, i2, amount, *p);
+  font_kern_list **p = kern_hash_table + hash_kern(g1, g2);
+  *p = new font_kern_list(g1, g2, amount, *p);
 }
 
-int font::get_kern(int i1, int i2, int point_size)
+int font::get_kern(glyph *g1, glyph *g2, int point_size)
 {
   if (kern_hash_table) {
-    for (font_kern_list *p = kern_hash_table[hash_kern(i1, i2)]; p; p = p->next)
-      if (i1 == p->i1 && i2 == p->i2)
+    for (font_kern_list *p = kern_hash_table[hash_kern(g1, g2)]; p;
+	 p = p->next)
+      if (g1 == p->glyph1 && g2 == p->glyph2)
 	return scale(p->amount, point_size);
   }
   return 0;
@@ -369,16 +529,42 @@ int font::has_ligature(int mask)
   return mask & ligatures;
 }
 
-int font::get_character_type(int c)
+int font::get_character_type(glyph *g)
 {
-  assert(c >= 0 && c < nindices && ch_index[c] >= 0);
-  return ch[ch_index[c]].type;
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  if (idx < nindices && ch_index[idx] >= 0) {
+    // Explicitly enumerated glyph
+    return ch[ch_index[idx]].type;
+  }
+  if (is_unicode) {
+    // Unicode font
+    return 0;
+  }
+  abort();
 }
 
-int font::get_code(int c)
+int font::get_code(glyph *g)
 {
-  assert(c >= 0 && c < nindices && ch_index[c] >= 0);
-  return ch[ch_index[c]].code;
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  if (idx < nindices && ch_index[idx] >= 0) {
+    // Explicitly enumerated glyph
+    return ch[ch_index[idx]].code;
+  }
+  if (is_unicode) {
+    // Unicode font
+    // ASCII or Unicode character, or groff glyph name that maps to Unicode?
+    int uni = glyph_to_unicode(g);
+    if (uni >= 0)
+      return uni;
+    // Numbered character?
+    int n = glyph_to_number(g);
+    if (n >= 0)
+      return n;
+  }
+  // The caller must check `contains(g)' before calling get_code(g).
+  abort();
 }
 
 const char *font::get_name()
@@ -391,10 +577,24 @@ const char *font::get_internal_name()
   return internalname;
 }
 
-const char *font::get_special_device_encoding(int c)
+const char *font::get_special_device_encoding(glyph *g)
 {
-  assert(c >= 0 && c < nindices && ch_index[c] >= 0);
-  return( ch[ch_index[c]].special_device_coding );
+  int idx = glyph_to_index(g);
+  assert(idx >= 0);
+  if (idx < nindices && ch_index[idx] >= 0) {
+    // Explicitly enumerated glyph
+    return ch[ch_index[idx]].special_device_coding;
+  }
+  if (is_unicode) {
+    // Unicode font
+    return NULL;
+  }
+  abort();
+}
+
+const char *font::get_image_generator()
+{
+  return image_generator;
 }
 
 void font::alloc_ch_index(int idx)
@@ -458,8 +658,9 @@ void font::compact()
   }
 }
 
-void font::add_entry(int idx, const font_char_metric &metric)
+void font::add_entry(glyph *g, const font_char_metric &metric)
 {
+  int idx = glyph_to_index(g);
   assert(idx >= 0);
   if (idx >= nindices)
     alloc_ch_index(idx);
@@ -471,18 +672,20 @@ void font::add_entry(int idx, const font_char_metric &metric)
   ch[ch_used++] = metric;
 }
 
-void font::copy_entry(int new_index, int old_index)
+void font::copy_entry(glyph *new_glyph, glyph *old_glyph)
 {
+  int new_index = glyph_to_index(new_glyph);
+  int old_index = glyph_to_index(old_glyph);
   assert(new_index >= 0 && old_index >= 0 && old_index < nindices);
   if (new_index >= nindices)
     alloc_ch_index(new_index);
   ch_index[new_index] = ch_index[old_index];
 }
 
-font *font::load_font(const char *s, int *not_found)
+font *font::load_font(const char *s, int *not_found, int head_only)
 {
   font *f = new font(s);
-  if (!f->load(not_found)) {
+  if (!f->load(not_found, head_only)) {
     delete f;
     return 0;
   }
@@ -557,8 +760,15 @@ again:
 // If the font can't be found, then if not_found is non-NULL, it will be set
 // to 1 otherwise a message will be printed.
 
-int font::load(int *not_found)
+int font::load(int *not_found, int head_only)
 {
+  if (strcmp(name, "DESC") == 0) {
+    if (not_found)
+      *not_found = 1;
+    else
+      error("`DESC' is not a valid font file name");
+    return 0;
+  }
   char *path;
   FILE *fp;
   if ((fp = open_file(name, &path)) == NULL) {
@@ -570,11 +780,12 @@ int font::load(int *not_found)
   }
   text_file t(fp, path);
   t.skip_comments = 1;
+  t.silent = head_only;
   char *p;
   for (;;) {
     if (!t.next()) {
-      t.error("missing charset command");
-      return 0;
+      p = 0;
+      break;
     }
     p = strtok(t.buf, WS);
     if (strcmp(p, "name") == 0) {
@@ -583,7 +794,7 @@ int font::load(int *not_found)
       p = strtok(0, WS);
       int n;
       if (p == 0 || sscanf(p, "%d", &n) != 1 || n <= 0) {
-	t.error("bad argument for spacewidth command");
+	t.error("bad argument for `spacewidth' command");
 	return 0;
       }
       space_width = n;
@@ -592,7 +803,7 @@ int font::load(int *not_found)
       p = strtok(0, WS);
       double n;
       if (p == 0 || sscanf(p, "%lf", &n) != 1 || n >= 90.0 || n <= -90.0) {
-	t.error("bad argument for slant command", p);
+	t.error("bad argument for `slant' command", p);
 	return 0;
       }
       slant = n;
@@ -621,7 +832,7 @@ int font::load(int *not_found)
     else if (strcmp(p, "internalname") == 0) {
       p = strtok(0, WS);
       if (!p) {
-	t.error("`internalname command requires argument");
+	t.error("`internalname' command requires argument");
 	return 0;
       }
       internalname = new char[strlen(p) + 1];
@@ -638,162 +849,168 @@ int font::load(int *not_found)
     else
       break;
   }
-  char *command = p;
   int had_charset = 0;
-  t.skip_comments = 0;
-  while (command) {
-    if (strcmp(command, "kernpairs") == 0) {
-      for (;;) {
-	if (!t.next()) {
-	  command = 0;
-	  break;
-	}
-	char *c1 = strtok(t.buf, WS);
-	if (c1 == 0)
-	  continue;
-	char *c2 = strtok(0, WS);
-	if (c2 == 0) {
-	  command = c1;
-	  break;
-	}
-	p = strtok(0, WS);
-	if (p == 0) {
-	  t.error("missing kern amount");
-	  return 0;
-	}
-	int n;
-	if (sscanf(p, "%d", &n) != 1) {
-	  t.error("bad kern amount `%1'", p);
-	  return 0;
-	}
-	int i1 = name_to_index(c1);
-	if (i1 < 0) {
-	  t.error("invalid character `%1'", c1);
-	  return 0;
-	}
-	int i2 = name_to_index(c2);
-	if (i2 < 0) {
-	  t.error("invalid character `%1'", c2);
-	  return 0;
-	}
-	add_kern(i1, i2, n);
-      }
+  if (p == 0) {
+    if (!is_unicode) {
+      t.error("missing charset command");
+      return 0;
     }
-    else if (strcmp(command, "charset") == 0) {
-      had_charset = 1;
-      int last_index = -1;
-      for (;;) {
-	if (!t.next()) {
-	  command = 0;
-	  break;
-	}
-	char *nm = strtok(t.buf, WS);
-	if (nm == 0)
-	  continue;			// I dont think this should happen
-	p = strtok(0, WS);
-	if (p == 0) {
-	  command = nm;
-	  break;
-	}
-	if (p[0] == '"') {
-	  if (last_index == -1) {
-	    t.error("first charset entry is duplicate");
-	    return 0;
+  } else {
+    char *command = p;
+    t.skip_comments = 0;
+    while (command) {
+      if (strcmp(command, "kernpairs") == 0) {
+	if (head_only)
+	  return 1;
+	for (;;) {
+	  if (!t.next()) {
+	    command = 0;
+	    break;
 	  }
-	  if (strcmp(nm, "---") == 0) {
-	    t.error("unnamed character cannot be duplicate");
-	    return 0;
-	  }
-	  int idx = name_to_index(nm);
-	  if (idx < 0) {
-	    t.error("invalid character `%1'", nm);
-	    return 0;
-	  }
-	  copy_entry(idx, last_index);
-	}
-	else {
-	  font_char_metric metric;
-	  metric.height = 0;
-	  metric.depth = 0;
-	  metric.pre_math_space = 0;
-	  metric.italic_correction = 0;
-	  metric.subscript_correction = 0;
-	  int nparms = sscanf(p, "%d,%d,%d,%d,%d,%d",
-			      &metric.width, &metric.height, &metric.depth,
-			      &metric.italic_correction,
-			      &metric.pre_math_space,
-			      &metric.subscript_correction);
-	  if (nparms < 1) {
-	    t.error("bad width for `%1'", nm);
-	    return 0;
+	  char *c1 = strtok(t.buf, WS);
+	  if (c1 == 0)
+	    continue;
+	  char *c2 = strtok(0, WS);
+	  if (c2 == 0) {
+	    command = c1;
+	    break;
 	  }
 	  p = strtok(0, WS);
 	  if (p == 0) {
-	    t.error("missing character type for `%1'", nm);
+	    t.error("missing kern amount");
 	    return 0;
 	  }
-	  int type;
-	  if (sscanf(p, "%d", &type) != 1) {
-	    t.error("bad character type for `%1'", nm);
+	  int n;
+	  if (sscanf(p, "%d", &n) != 1) {
+	    t.error("bad kern amount `%1'", p);
 	    return 0;
 	  }
-	  if (type < 0 || type > 255) {
-	    t.error("character type `%1' out of range", type);
-	    return 0;
+	  glyph *g1 = name_to_glyph(c1);
+	  glyph *g2 = name_to_glyph(c2);
+	  add_kern(g1, g2, n);
+	}
+      }
+      else if (strcmp(command, "charset") == 0) {
+	if (head_only)
+	  return 1;
+	had_charset = 1;
+	glyph *last_glyph = NULL;
+	for (;;) {
+	  if (!t.next()) {
+	    command = 0;
+	    break;
 	  }
-	  metric.type = type;
+	  char *nm = strtok(t.buf, WS);
+	  if (nm == 0)
+	    continue;			// I dont think this should happen
 	  p = strtok(0, WS);
 	  if (p == 0) {
-	    t.error("missing code for `%1'", nm);
-	    return 0;
+	    command = nm;
+	    break;
 	  }
-	  char *ptr;
-	  metric.code = (int)strtol(p, &ptr, 0);
-	  if (metric.code == 0 && ptr == p) {
-	    t.error("bad code `%1' for character `%2'", p, nm);
-	    return 0;
-	  }
-	  p = strtok(0, WS);
-	  if ((p == NULL) || (strcmp(p, "--") == 0)) {
-	    metric.special_device_coding = NULL;
-	  }
-	  else {
-	    char *nam = new char[strlen(p) + 1];
-	    strcpy(nam, p);
-	    metric.special_device_coding = nam;
-	  }
-	  if (strcmp(nm, "---") == 0) {
-	    last_index = number_to_index(metric.code);
-	    add_entry(last_index, metric);
-	  }
-	  else {
-	    last_index = name_to_index(nm);
-	    if (last_index < 0) {
-	      t.error("invalid character `%1'", nm);
+	  if (p[0] == '"') {
+	    if (last_glyph == NULL) {
+	      t.error("first charset entry is duplicate");
 	      return 0;
 	    }
-	    add_entry(last_index, metric);
-	    copy_entry(number_to_index(metric.code), last_index);
+	    if (strcmp(nm, "---") == 0) {
+	      t.error("unnamed character cannot be duplicate");
+	      return 0;
+	    }
+	    glyph *g = name_to_glyph(nm);
+	    copy_entry(g, last_glyph);
+	  }
+	  else {
+	    font_char_metric metric;
+	    metric.height = 0;
+	    metric.depth = 0;
+	    metric.pre_math_space = 0;
+	    metric.italic_correction = 0;
+	    metric.subscript_correction = 0;
+	    int nparms = sscanf(p, "%d,%d,%d,%d,%d,%d",
+				&metric.width, &metric.height, &metric.depth,
+				&metric.italic_correction,
+				&metric.pre_math_space,
+				&metric.subscript_correction);
+	    if (nparms < 1) {
+	      t.error("bad width for `%1'", nm);
+	      return 0;
+	    }
+	    p = strtok(0, WS);
+	    if (p == 0) {
+	      t.error("missing character type for `%1'", nm);
+	      return 0;
+	    }
+	    int type;
+	    if (sscanf(p, "%d", &type) != 1) {
+	      t.error("bad character type for `%1'", nm);
+	      return 0;
+	    }
+	    if (type < 0 || type > 255) {
+	      t.error("character type `%1' out of range", type);
+	      return 0;
+	    }
+	    metric.type = type;
+	    p = strtok(0, WS);
+	    if (p == 0) {
+	      t.error("missing code for `%1'", nm);
+	      return 0;
+	    }
+	    char *ptr;
+	    metric.code = (int)strtol(p, &ptr, 0);
+	    if (metric.code == 0 && ptr == p) {
+	      t.error("bad code `%1' for character `%2'", p, nm);
+	      return 0;
+	    }
+	    if (is_unicode) {
+	      int w = wcwidth(metric.code);
+	      if (w > 1)
+	        metric.width *= w;
+	    }
+	    p = strtok(0, WS);
+	    if ((p == NULL) || (strcmp(p, "--") == 0)) {
+	      metric.special_device_coding = NULL;
+	    }
+	    else {
+	      char *nam = new char[strlen(p) + 1];
+	      strcpy(nam, p);
+	      metric.special_device_coding = nam;
+	    }
+	    if (strcmp(nm, "---") == 0) {
+	      last_glyph = number_to_glyph(metric.code);
+	      add_entry(last_glyph, metric);
+	    }
+	    else {
+	      last_glyph = name_to_glyph(nm);
+	      add_entry(last_glyph, metric);
+	      copy_entry(number_to_glyph(metric.code), last_glyph);
+	    }
 	  }
 	}
+	if (last_glyph == NULL) {
+	  t.error("I didn't seem to find any characters");
+	  return 0;
+	}
       }
-      if (last_index == -1) {
-	t.error("I didn't seem to find any characters");
+      else {
+	t.error("unrecognised command `%1' "
+		"after `kernpairs' or `charset' command",
+		  command);
 	return 0;
       }
     }
-    else {
-      t.error("unrecognised command `%1' after `kernpairs' or `charset' command", command);
-      return 0;
-    }
+    compact();
   }
-  if (!had_charset) {
-    t.error("missing charset command");
+  if (!is_unicode && !had_charset) {
+    t.error("missing `charset' command");
     return 0;
   }
-  if (space_width == 0)
-    space_width = scale_round(unitwidth, res, 72*3*sizescale);
-  compact();
+  if (space_width == 0) {
+    if (zoom)
+      space_width = scale_round(unitwidth, res, 72 * 3 * sizescale, zoom);
+    else
+      space_width = scale_round(unitwidth, res, 72 * 3 * sizescale);
+  }
   return 1;
 }
 
@@ -810,7 +1027,7 @@ static struct {
   { "spare1", &font::biggestfont },
   { "biggestfont", &font::biggestfont },
   { "spare2", &font::spare2 },
-  { "sizescale", &font::sizescale }
+  { "sizescale", &font::sizescale },
   };
 
 int font::load_desc()
@@ -905,6 +1122,8 @@ int font::load_desc()
 	return 0;
       }
     }
+    else if (strcmp("unscaled_charwidths", p) == 0)
+      unscaled_charwidths = 1;
     else if (strcmp("pass_filenames", p) == 0)
       pass_filenames = 1;
     else if (strcmp("sizes", p) == 0) {
@@ -981,6 +1200,16 @@ int font::load_desc()
       tcommand = 1;
     else if (strcmp("use_charnames_in_special", p) == 0)
       use_charnames_in_special = 1;
+    else if (strcmp("unicode", p) == 0)
+      is_unicode = 1;
+    else if (strcmp("image_generator", p) == 0) {
+      p = strtok(0, WS);
+      if (!p) {
+	t.error("image_generator command requires an argument");
+	return 0;
+      }
+      image_generator = strsave(p);
+    }
     else if (strcmp("charset", p) == 0)
       break;
     else if (unknown_desc_command_handler) {
