@@ -6,8 +6,8 @@
  * gremlin picture editor.
  */
 
-#include "gprint.h"
 #include <stdlib.h>
+#include "gprint.h"
 #include <string.h>
 #include <ctype.h>
 
@@ -22,6 +22,7 @@
 extern int linenum;		/* current line number in input file */
 extern char gremlinfile[];	/* name of file currently reading */
 extern int SUNFILE;		/* TRUE if SUN gremlin file */
+extern int compatibility_flag;	/* TRUE if in compatibility mode */
 extern void savebounds(double x, double y);
 
 /* imports from hpoint.cpp */
@@ -146,6 +147,10 @@ DBRead(register FILE *file)
 	    (void) sscanf(string, "%lf%lf", &x, &y);
 	    if ((x == -1.00 && y == -1.00) && (!SUNFILE))
 	      lastpoint = TRUE;
+	    else {
+	      if (compatibility_flag)
+		savebounds(xorn(x, y), yorn(x, y));
+	    }
 	  }
 	} while (!lastpoint);
 #endif	/* UW_FASTSCAN */
