@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992, 2001, 2003
+/* Copyright (C) 1989, 1990, 1991, 1992, 2001, 2003, 2008, 2009
    Free Software Foundation, Inc.
 Written by James Clark (jjc@jclark.com)
 
@@ -7,17 +7,16 @@ This file is part of groff.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2, or (at your option) any later
-version.
+Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 groff is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
-You should have received a copy of the GNU General Public License along
-with groff; see the file COPYING.  If not, write to the Free Software
-Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+You should have received a copy of the GNU General Public License
+along with this program. If not, see <http://www.gnu.org/licenses/>. */
      
 #include "refer.h"
 #include "refid.h"
@@ -420,8 +419,11 @@ void reference::compute_sort_key()
     return;
   sort_fields += '\0';
   const char *sf = sort_fields.contents();
+  int first_time = 1;
   while (*sf != '\0') {
-    sort_key += SORT_SEP;
+    if (!first_time)
+      sort_key += SORT_SEP;
+    first_time = 0;
     char f = *sf++;
     int n = 1;
     if (*sf == '+') {
