@@ -1,5 +1,6 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001, 2002, 2003, 2004
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001, 2002, 2003, 2004,\
+                 2006, 2009, 2010
    Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
@@ -7,17 +8,16 @@ This file is part of groff.
 
 groff is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2, or (at your option) any later
-version.
+Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 groff is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
-You should have received a copy of the GNU General Public License along
-with groff; see the file COPYING.  If not, write to the Free Software
-Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+You should have received a copy of the GNU General Public License
+along with this program. If not, see <http://www.gnu.org/licenses/>. */
 
 struct hyphen_list {
   unsigned char hyphen;
@@ -50,14 +50,21 @@ class token_node;
 struct node {
   node *next;
   node *last;
+  statem *state;
+  statem *push_state;
+  int div_nest_level;
+  int is_special;
   node();
-  node(node *n);
-  node *add_char(charinfo *c, environment *, hunits *widthp, int *spacep);
+  node(node *);
+  node(node *, statem *, int);
+  node *add_char(charinfo *, environment *, hunits *, int *, node ** = 0);
 
   virtual ~node();
   virtual node *copy() = 0;
   virtual int set_unformat_flag();
   virtual int force_tprint() = 0;
+  virtual int is_tag() = 0;
+  virtual int get_break_code();
   virtual hunits width();
   virtual hunits subscript_correction();
   virtual hunits italic_correction();
@@ -67,7 +74,7 @@ struct node {
   virtual int merge_space(hunits, hunits, hunits);
   virtual vunits vertical_width();
   virtual node *last_char_node();
-  virtual void vertical_extent(vunits *min, vunits *max);
+  virtual void vertical_extent(vunits *, vunits *);
   virtual int character_type();
   virtual void set_vertical_size(vertical_size *);
   virtual int ends_sentence();
@@ -81,9 +88,7 @@ struct node {
   virtual void spread_space(int *, hunits *);
   virtual void freeze_space();
   virtual void is_escape_colon();
-  virtual breakpoint *get_breakpoints(hunits width, int nspaces,
-				      breakpoint *rest = 0,
-				      int is_inner = 0);
+  virtual breakpoint *get_breakpoints(hunits, int, breakpoint * = 0, int = 0);
   virtual int nbreaks();
   virtual void split(int, node **, node **);
   virtual hyphenation_type get_hyphenation_type();
@@ -105,20 +110,35 @@ struct node {
 
   virtual int same(node *) = 0;
   virtual const char *type() = 0;
+  virtual void debug_node();
+  virtual void debug_node_list();
 };
 
 inline node::node()
-: next(0), last(0)
+: next(0), last(0), state(0), push_state(0), div_nest_level(0), is_special(0)
 {
 }
 
 inline node::node(node *n)
-: next(n), last(0)
+: next(n), last(0), state(0), push_state(0), div_nest_level(0), is_special(0)
 {
 }
 
+inline node::node(node *n, statem *s, int divlevel)
+: next(n), last(0), push_state(0), div_nest_level(divlevel), is_special(0)
+{
+  if (s)
+    state = new statem(s);
+  else
+    state = 0;
+}
+
 inline node::~node()
 {
+  if (state != 0)
+    delete state;
+  if (push_state != 0)
+    delete push_state;
 }
 
 // 0 means it doesn't, 1 means it does, 2 means it's transparent
@@ -140,6 +160,7 @@ public:
   node *copy() { return new line_start_node; }
   int same(node *);
   int force_tprint();
+  int is_tag();
   const char *type();
   void asciify(macro *);
 };
@@ -156,7 +177,7 @@ protected:
   char set;
   char was_escape_colon;
   color *col;			/* for grotty */
-  space_node(hunits, int, int, color *, node * = 0);
+  space_node(hunits, int, int, color *, statem *, int, node * = 0);
 public:
   space_node(hunits, color *, node * = 0);
 #if 0
@@ -172,8 +193,7 @@ public:
   void is_escape_colon();
   void spread_space(int *, hunits *);
   void tprint(troff_output_file *);
-  breakpoint *get_breakpoints(hunits width, int nspaces, breakpoint *rest = 0,
-			      int is_inner = 0);
+  breakpoint *get_breakpoints(hunits, int, breakpoint * = 0, int = 0);
   int nbreaks();
   void split(int, node **, node **);
   void ascii_print(ascii_output_file *);
@@ -181,6 +201,7 @@ public:
   void asciify(macro *);
   const char *type();
   int force_tprint();
+  int is_tag();
   hyphenation_type get_hyphenation_type();
 };
 
@@ -196,7 +217,8 @@ class word_space_node : public space_node {
 protected:
   width_list *orig_width;
   unsigned char unformat;
-  word_space_node(hunits, int, color *, width_list *, int, node * = 0);
+  word_space_node(hunits, int, color *, width_list *, int, statem *, int,
+		  node * = 0);
 public:
   word_space_node(hunits, color *, width_list *, node * = 0);
   ~word_space_node();
@@ -209,10 +231,11 @@ public:
   const char *type();
   int merge_space(hunits, hunits, hunits);
   int force_tprint();
+  int is_tag();
 };
 
 class unbreakable_space_node : public word_space_node {
-  unbreakable_space_node(hunits, int, color *, node * = 0);
+  unbreakable_space_node(hunits, int, color *, statem *, int, node * = 0);
 public:
   unbreakable_space_node(hunits, color *, node * = 0);
   node *copy();
@@ -222,8 +245,8 @@ public:
   void asciify(macro *);
   const char *type();
   int force_tprint();
-  breakpoint *get_breakpoints(hunits width, int nspaces, breakpoint *rest = 0,
-			      int is_inner = 0);
+  int is_tag();
+  breakpoint *get_breakpoints(hunits, int, breakpoint * = 0, int = 0);
   int nbreaks();
   void split(int, node **, node **);
   int merge_space(hunits, hunits, hunits);
@@ -235,41 +258,48 @@ public:
 class diverted_space_node : public node {
 public:
   vunits n;
-  diverted_space_node(vunits d, node *p = 0);
+  diverted_space_node(vunits, node * = 0);
+  diverted_space_node(vunits, statem *, int, node * = 0);
   node *copy();
   int reread(int *);
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 };
 
 class diverted_copy_file_node : public node {
   symbol filename;
 public:
   vunits n;
-  diverted_copy_file_node(symbol s, node *p = 0);
+  diverted_copy_file_node(symbol, node * = 0);
+  diverted_copy_file_node(symbol, statem *, int, node * = 0);
   node *copy();
   int reread(int *);
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 };
 
 class extra_size_node : public node {
   vunits n;
 public:
-  extra_size_node(vunits i) : n(i) {}
+  extra_size_node(vunits);
+  extra_size_node(vunits, statem *, int);
   void set_vertical_size(vertical_size *);
   node *copy();
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 };
 
 class vertical_size_node : public node {
   vunits n;
 public:
-  vertical_size_node(vunits i) : n(i) {}
+  vertical_size_node(vunits, statem *, int);
+  vertical_size_node(vunits);
   void set_vertical_size(vertical_size *);
   void asciify(macro *);
   node *copy();
@@ -277,6 +307,7 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 };
 
 class hmotion_node : public node {
@@ -288,6 +319,12 @@ protected:
 public:
   hmotion_node(hunits i, color *c, node *nxt = 0)
     : node(nxt), n(i), was_tab(0), unformat(0), col(c) {}
+  hmotion_node(hunits i, color *c, statem *s, int divlevel, node *nxt = 0)
+    : node(nxt, s, divlevel), n(i), was_tab(0), unformat(0), col(c) {}
+  hmotion_node(hunits i, int flag1, int flag2, color *c, statem *s,
+	       int divlevel, node *nxt = 0)
+    : node(nxt, s, divlevel), n(i), was_tab(flag1), unformat(flag2),
+      col(c) {}
   hmotion_node(hunits i, int flag1, int flag2, color *c, node *nxt = 0)
     : node(nxt), n(i), was_tab(flag1), unformat(flag2), col(c) {}
   node *copy();
@@ -300,6 +337,7 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
   node *add_self(node *, hyphen_list **);
   hyphen_list *get_hyphen_list(hyphen_list *, int *);
   hyphenation_type get_hyphenation_type();
@@ -308,6 +346,7 @@ public:
 class space_char_hmotion_node : public hmotion_node {
 public:
   space_char_hmotion_node(hunits, color *, node * = 0);
+  space_char_hmotion_node(hunits, color *, statem *, int, node * = 0);
   node *copy();
   void ascii_print(ascii_output_file *);
   void asciify(macro *);
@@ -315,6 +354,7 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
   node *add_self(node *, hyphen_list **);
   hyphen_list *get_hyphen_list(hyphen_list *, int *);
   hyphenation_type get_hyphenation_type();
@@ -324,20 +364,23 @@ class vmotion_node : public node {
   vunits n;
   color *col;			/* for grotty */
 public:
-  vmotion_node(vunits i, color *c) : n(i), col(c) {}
+  vmotion_node(vunits, color *);
+  vmotion_node(vunits, color *, statem *, int);
   void tprint(troff_output_file *);
   node *copy();
   vunits vertical_width();
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 };
 
 class hline_node : public node {
   hunits x;
   node *n;
 public:
-  hline_node(hunits i, node *c, node *nxt = 0) : node(nxt), x(i), n(c) {}
+  hline_node(hunits, node *, node * = 0);
+  hline_node(hunits, node *, statem *, int, node * = 0);
   ~hline_node();
   node *copy();
   hunits width();
@@ -345,13 +388,15 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 };
 
 class vline_node : public node {
   vunits x;
   node *n;
 public:
-  vline_node(vunits i, node *c, node *nxt = 0) : node(nxt), x(i), n(c) {}
+  vline_node(vunits, node *, node * = 0);
+  vline_node(vunits, node *, statem *, int, node * = 0);
   ~vline_node();
   node *copy();
   void tprint(troff_output_file *);
@@ -361,9 +406,9 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 };
 
-
 class dummy_node : public node {
 public:
   dummy_node(node *nd = 0) : node(nd) {}
@@ -371,6 +416,7 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
   hyphenation_type get_hyphenation_type();
 };
 
@@ -381,6 +427,7 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
   int ends_sentence();
   hyphenation_type get_hyphenation_type();
 };
@@ -388,16 +435,18 @@ public:
 class zero_width_node : public node {
   node *n;
 public:
-  zero_width_node(node *gn);
+  zero_width_node(node *);
+  zero_width_node(node *, statem *, int);
   ~zero_width_node();
   node *copy();
   void tprint(troff_output_file *);
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
   void append(node *);
   int character_type();
-  void vertical_extent(vunits *min, vunits *max);
+  void vertical_extent(vunits *, vunits *);
 };
 
 class left_italic_corrected_node : public node {
@@ -405,6 +454,7 @@ class left_italic_corrected_node : public node {
   hunits x;
 public:
   left_italic_corrected_node(node * = 0);
+  left_italic_corrected_node(statem *, int, node * = 0);
   ~left_italic_corrected_node();
   void tprint(troff_output_file *);
   void ascii_print(ascii_output_file *);
@@ -413,6 +463,7 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
   hunits width();
   node *last_char_node();
   void vertical_extent(vunits *, vunits *);
@@ -435,6 +486,7 @@ class overstrike_node : public node {
   hunits max_width;
 public:
   overstrike_node();
+  overstrike_node(statem *, int);
   ~overstrike_node();
   node *copy();
   void tprint(troff_output_file *);
@@ -443,6 +495,7 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
   node *add_self(node *, hyphen_list **);
   hyphen_list *get_hyphen_list(hyphen_list *, int *);
   hyphenation_type get_hyphenation_type();
@@ -453,6 +506,7 @@ class bracket_node : public node {
   hunits max_width;
 public:
   bracket_node();
+  bracket_node(statem *, int);
   ~bracket_node();
   node *copy();
   void tprint(troff_output_file *);
@@ -461,6 +515,7 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 };
 
 class special_node : public node {
@@ -474,12 +529,14 @@ class special_node : public node {
   void tprint_end(troff_output_file *);
 public:
   special_node(const macro &, int = 0);
-  special_node(const macro &, tfont *, color *, color *, int = 0);
+  special_node(const macro &, tfont *, color *, color *, statem *, int,
+	       int = 0);
   node *copy();
   void tprint(troff_output_file *);
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
   int ends_sentence();
   tfont *get_tfont();
 };
@@ -492,16 +549,34 @@ class suppress_node : public node {
   int  image_id;
 public:
   suppress_node(int, int);
-  suppress_node(symbol f, char p, int id);
-  suppress_node(int, int, symbol f, char p, int id);
+  suppress_node(symbol, char, int);
+  suppress_node(int, int, symbol, char, int, statem *, int);
+  suppress_node(int, int, symbol, char, int);
   node *copy();
   void tprint(troff_output_file *);
   hunits width();
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 private:
-  void put(troff_output_file *out, const char *s);
+  void put(troff_output_file *, const char *);
+};
+
+class tag_node : public node {
+public:
+  string tag_string;
+  int delayed;
+  tag_node();
+  tag_node(string, int);
+  tag_node(string, statem *, int, int);
+  node *copy();
+  void tprint(troff_output_file *);
+  int same(node *);
+  const char *type();
+  int force_tprint();
+  int is_tag();
+  int ends_sentence();
 };
 
 struct hvpair {
@@ -519,6 +594,7 @@ class draw_node : public node {
   hvpair *point;
 public:
   draw_node(char, hvpair *, int, font_size, color *, color *);
+  draw_node(char, hvpair *, int, font_size, color *, color *, statem *, int);
   ~draw_node();
   hunits width();
   vunits vertical_width();
@@ -527,18 +603,19 @@ public:
   int same(node *);
   const char *type();
   int force_tprint();
+  int is_tag();
 };
 
 class charinfo;
-node *make_node(charinfo *ci, environment *);
+node *make_node(charinfo *, environment *);
 int character_exists(charinfo *, environment *);
 
-int same_node_list(node *n1, node *n2);
-node *reverse_node_list(node *n);
+int same_node_list(node *, node *);
+node *reverse_node_list(node *);
 void delete_node_list(node *);
 node *copy_node_list(node *);
 
-int get_bold_fontno(int f);
+int get_bold_fontno(int);
 
 inline hyphen_list::hyphen_list(unsigned char code, hyphen_list *p)
 : hyphen(0), breakable(0), hyphenation_code(code), next(p)
@@ -546,9 +623,11 @@ inline hyphen_list::hyphen_list(unsigned char code, hyphen_list *p)
 }
 
 extern void read_desc();
-extern int mount_font(int n, symbol, symbol = NULL_SYMBOL);
-extern void mount_style(int n, symbol);
-extern int is_good_fontno(int n);
+extern int mount_font(int, symbol, symbol = NULL_SYMBOL);
+extern int check_font(symbol, symbol);
+extern int check_style(symbol);
+extern void mount_style(int, symbol);
+extern int is_good_fontno(int);
 extern int symbol_fontno(symbol);
 extern int next_available_font_position();
 extern void init_size_table(int *);
@@ -559,7 +638,7 @@ class output_file {
 public:
   output_file();
   virtual ~output_file();
-  virtual void trailer(vunits page_length);
+  virtual void trailer(vunits);
   virtual void flush() = 0;
   virtual void transparent_char(unsigned char) = 0;
   virtual void print_line(hunits x, vunits y, node *n,
@@ -567,12 +646,13 @@ public:
   virtual void begin_page(int pageno, vunits page_length) = 0;
   virtual void copy_file(hunits x, vunits y, const char *filename) = 0;
   virtual int is_printing() = 0;
-  virtual void put_filename(const char *filename);
+  virtual void put_filename(const char *, int);
   virtual void on();
   virtual void off();
 #ifdef COLUMN
   virtual void vjustify(vunits, symbol);
 #endif /* COLUMN */
+  mtsm state;
 };
 
 #ifndef POPEN_MISSING
@@ -581,7 +661,7 @@ extern char *pipe_command;
 
 extern output_file *the_output;
 extern void init_output();
-int in_output_page_list(int n);
+int in_output_page_list(int);
 
 class font_family {
   int *map;
@@ -596,4 +676,5 @@ public:
 
 font_family *lookup_family(symbol);
 symbol get_font_name(int, environment *);
+symbol get_style_name(int);
 extern search_path include_search_path;
